; +-----------------------+------------------+
; |      Loonadrift       |    Ryan Touk     |
; +-----------------------+------------------+
; |  For Ludum Dare #20:                     |
; | "It's Dangerous to go Alone! Take this!" |
; +------------------------------------------+
; |  4.29.2011 . Creation (PureBasic 4.51)
; |  5.01.     - Version 1.0, 48-Hour competition version
; |   .02.     - Version 1.1, 72-Hour jam version
; |                Made death less abrupt, distance more apparent
; |                Added double-tap speed feature, health every 25 kills
; |   .03.     - Version 1.2
; |                Balloon color selection, Mac compatible
; |   .04.     - Version 1.3
; |                Fixed always-report-record bug, minor tweaks


; TO-MAYBE-DO
; difficulties ?
; sprite magenta -> true png transparency ?

; BUGS
; generated throwers fail if distances is too short ?






;-
;- Constants

#BG_Title   = "Loonadrift"
#BG_Author  = "Ryan Touk"
#BG_Version = "1.3"

#BG_BaseW  =  640
#BG_BaseH  =  480
#BG_BaseD  =   32
#BG_DefWin = #True

#BG_StartX = 693.0
#BG_StartY = 256.0

#BG_SpringPhys   = #True
#BG_SpringNodes  =  3
#BG_SpringLength =  5
#BG_SpringConst  =  0.8
#BG_SpringGrav   =  2.0

#BG_SaveFile = "balloon.sav"

#BG_MaxSpeed = 10.0
#BG_TapTime = 300

#BG_Music    = "theme.ogg"
#BG_Music2   = "eog.ogg"

#BG_RedSize = 64

#BG_TransColor = $FF00FF
#Black         = $000000
#White         = $FFFFFF

#BG_DataEnd = ";END"

#BG_BPM     = 120
#BG_BeatLen = 60 * 1000 / #BG_BPM

; Object Flags

#BG_Popup    = %000000000001
#BG_Flying   = %000000000010
#BG_Dead     = %000000000100
#BG_Friend   = %000000001000
#BG_Pulsy    = %000000010000
#BG_Newton   = %000000100000
#BG_Awake    = %000001000000
#BG_Weapon   = %000010000000
#BG_Bad      = %000100000000
#BG_Volatile = %001000000000
#BG_Gen      = %010000000000
#BG_RPopup   = %100000000000

CompilerIf (#PB_Compiler_OS = #PB_OS_Windows)
  #BG_Error      = #MB_ICONHAND
  #BG_MusicFlags = #PB_Sound_Streaming
  #BG_HearPath   = "./hear/"
CompilerElse
  #BG_Error      = #Null
  #BG_MusicFlags = #Null
  #BG_HearPath   = "./hearm/"
  CompilerIf (#PB_Compiler_OS = #PB_OS_Linux)
    CompilerError "Linux version is untested! (You can comment out this warning.)"
  CompilerEndIf
CompilerEndIf

Enumeration
  #BG_Reddy
  #BG_Reddy0
EndEnumeration

Enumeration
  #BG_3DReddy
EndEnumeration








;- Structures

Structure STRINGNODE
  x.f
  y.f
  xv.f
  yv.f
EndStructure

Structure BUILDING
  Color.i
  x.f
  w.f
  h.f
  z.f
EndStructure

Structure OBCLASS
  Flags.i
  xv.f
  
  hoveramount.f
  StructureUnion
    hoverspeed.f
    gravity.f
  EndStructureUnion
  
  StructureUnion
    popspeed.f
    risespeed.f
  EndStructureUnion
  
  Frames.i
  FrameNames.s
  FrameTicks.i
  
  oleft.f
  oright.f
  otop.f
  obottom.f
  hide.f
  
  Weapon.i
  
  AppearSound.i
  DieSound.i
  HitSound.i
  
  HP.i
  
  Size.i
EndStructure

Structure OBSTACLE
  sx.f
  sy.f
  x.f
  y.f
  ty.f
  py.f
  yv.f
  
  Scale.f
  
  Flags.i
  
  hovera.f
  Frame.i
  OFrame.i
  FTime.i
  
  Mess.s
  
  HP.i
  
  Class.i
EndStructure

Structure OBFRAME
  Sprite2D.i
  Sprite3D.i
  Size.i
EndStructure

Structure BULLET
  x.f
  y.f
  xv.f
  yv.f
  Weapon.i
  Hits.i
  OFrame.i
  Frame.i
  FTime.i
  a.f
EndStructure

Structure WEAPON
  Rate.i
  Ammo.i
  BSpeed.f
  Limit.i
  
  Frames.i
  FrameNames.s
  FrameTicks.i
  OFrame.i
  
  Damage.i
  
  Gravity.f
  Recoil.f
  Hits.i
  Rand.i
  
  GetSound.i
  ShootSound.i
EndStructure







;- Variables

EnableExplicit

Global ScreenW.i  = #BG_BaseW
Global ScreenH.i  = #BG_BaseH
Global ScreenD.i  = #BG_Based
Global Windowed.i = #BG_DefWin
Global InScreen.i = #False
Global ExitFlag.i = #False

Global MyColor.i = -1, ospkey.i

Global JoyInit.i

Global Tries.i = 0, IntroTime.i, DeadTime.i

Global Reach.f
Global RecordReach.f = 0.0
Global Record.i
Global StartX.f

Global SoundInit.i
Global SoundOn.i, MusicOn.i
Global BG_MusicLen.i = 96050

Global SurviveY.i

Global CamX.f  = 0.0
Global CamTX.f = 0.0
Global CamV.f  = 0.0
Global CamTV.f = 0.0

Global LastXKey.i = 0, LastXTime.i = 0, DoubleX.f = 1.0
Global LastYKey.i = 0, LastYTime.i = 0, DoubleY.f = 1.0

Global Kills.i

Global GenX.f = 0.0

Global Health.i, InIntro.i, Dead.i

Global LastThrow.f

Global XScale.f, YScale.f

Global Level.i
Global Hurt.i

Global px.f, py.f
Global pxv.f, pyv.f
Global pa.f, drifta.f
Global bx.f, by.f

Global PopSound.i, RecSound.i, SadSound.i, WindSound.i, TypeSound.i
Global Woo.i, Wahoo.i, Ohh.i

Global MinX.f, MaxX.f
Global MinY.f, MaxY.f

Global FFont.i

Global Immortal.i = #False

Global BeatPhase.f
Global BeatTime.i, Time.i

Global sox.f, soy.f, sor.f

Global BackR.i, BackG.i, BackB.i

Global ospace.i, space.i

Define *OC.OBCLASS, *OF.OBFRAME, *W.WEAPON
Define.s St, St2
Define.f x, y, dx, dy, d, a, f
Define.i i, j, k, n, Event, ID

Global Message.s, MessageLen.i, MessageN.i, MessageTime.i

Global NewMap OC.OBCLASS()

Global NewList BC.i()
Global NewList WL.i()

Global NewList O.OBSTACLE()

Global NewList BD.BUILDING()

Global NewMap OF.OBFRAME()

Global ShootTime.i, MyAmmo.i
Global *MyWeapon.WEAPON = #Null
Global Weapons.i = 0
Global NewMap W.WEAPON()

Global NewList B.BULLET()

Global NewMap S.i()

CompilerIf (#BG_SpringPhys)
  Global NewList SN.STRINGNODE()
  Global Nodes.i
CompilerEndIf

#BG_Shouts = 12
Global Dim Shout.s(#BG_Shouts - 1)
  Shout(0) = "Take this too!"
  Shout(1) = "Try this!"
  Shout(2) = "This might help!"
  Shout(3) = "Arbitrary Balloon Units!"
  Shout(4) = "Go get 'em!"
  Shout(5) = "Godspeed!"
  Shout(6) = "Be careful!"
  Shout(7) = "I'll never forget you!"
  Shout(8) = "Go for it!"
  Shout(9) = "Yeah!"
  Shout(10) = "Keep going!"
  Shout(11) = "You can do it!"




;-
;- Macros

Macro ScreenX(wx)
  (((wx) - CamX) * XScale)
EndMacro

Macro ScreenY(wy)
  (ScreenH - ((wy) * YScale))
EndMacro

Macro CueSound(Num)
  If (SoundOn)
    PlaySound(Num)
  EndIf
EndMacro

Macro CueMusic()
  If (SoundInit)
    PlaySound(0)
    BeatTime = ElapsedMilliseconds()
  EndIf
EndMacro

Macro CueMessage(Text)
  Message     = Text
  MessageLen  = Len(Message)
  MessageN    = 1
  MessageTime = Time + 1000 + MessageLen * 150
  CueSound(PopSound)
EndMacro








;-
;- Procedures

Procedure.s Hash(Dist.s)
  Static In.s, Out.s
  RandomSeed(1337)
  In = Dist + Str(Random(#White))
  Out = Mid(MD5Fingerprint(@In, Len(In)), 2, 30)
  In = ""
  RandomSeed(ElapsedMilliseconds())
  ProcedureReturn Out
EndProcedure

Procedure Shutdown(Toggle.i = #False)
  If (InScreen)
    CloseScreen()
  EndIf

  If (IsWindow(0))
    CloseWindow(0)
  EndIf

  If (CreateFile(0, #BG_SaveFile))
    WriteStringN(0, StrF(RecordReach, 1) + " ABU")
    WriteStringN(0, Hash(StrF(RecordReach, 1)))
    If (MyColor >= 0)
      WriteStringN(0, RSet(Hex(RGB(Blue(MyColor),Green(MyColor),Red(MyColor))), 6, "0"))
    EndIf
    CloseFile(0)
  EndIf
  
  If (Toggle)
    If (Windowed)
      RunProgram(ProgramFilename(), "-f", "")
    Else
      RunProgram(ProgramFilename(), "-w", "")
    EndIf
  EndIf

  End
EndProcedure

Procedure Error(Message.s)
  MessageRequester(#BG_Title, Message, #BG_Error)
  Shutdown()
EndProcedure

Procedure ResetLevel()
  Static x.f, y.f, *OC.OBCLASS, i.i
  
  ;ResetMap(W())
  ;NextMapElement(W())
  ;*MyWeapon = @W()
  *MyWeapon = #Null
  
  CamX  = 200.0
  CamTX = 200.0
  CamV  = 0.0
  CamTV = 2.0
  
  BackR = 255
  BackG = 255
  BackB = 255
  
  LastThrow = 0.0
  
  Kills = 0
  
  Message = ""
  
  If (Record)
    CueSound(RecSound)
  Else
    CueSound(SadSound)
  EndIf
  
  Reach.f = 0.0
  Record.i = #False
  
  SurviveY = -#BG_BaseH / 3
  
  ;py     = #BG_BaseH * 0.50 + SpriteHeight(#BG_Reddy) * YScale
  px = #BG_StartX
  py = #BG_StartY
  pxv = 0.0
  pyv = 0.0
  pa  = 0.0
  drifta = 0.0
  InIntro = #True
  If (SoundInit And IsSound(0))
    StopSound(0)
  EndIf
  
  Hurt = #False
  
  Health =  10
  Dead   = #False
  
  CompilerIf (#BG_SpringPhys)
    ClearList(SN())
    AddElement(SN())
      SN()\x = px + 7.0 + sox
      SN()\y = py - #BG_RedSize
      x = SN()\x
      y = SN()\y
    Nodes = 0
    For i = 1 To (Health-1) * #BG_SpringNodes
      AddElement(SN())
        SN()\x  = x; + Random(2 * #BG_SpringLength) - #BG_SpringLength
        SN()\y  = y - #BG_SpringGrav/#BG_SpringConst - #BG_SpringLength
        SN()\xv = 0.0
        SN()\yv = 0.0
        y = SN()\y
        Nodes + 1
    Next i
  CompilerEndIf
  
  x = 0.0
  ForEach BD()
    BD()\Color = RGB(Random(192),Random(192),Random(192))
    BD()\x = x + BD()\w + 100.0 + Random(400)
    x = BD()\x
  Next
  
  ForEach O()
    If (O()\Flags & #BG_Gen)
      DeleteElement(O())
    Else
      *OC = O()\Class
      O()\x = O()\sx
      If O()\Flags & #BG_Newton
        O()\yv = *OC\risespeed
      EndIf
      O()\HP = *OC\HP
      O()\py = O()\sy
      O()\y = O()\py
      O()\Flags & (~(#BG_Dead|#BG_Awake))
      O()\Frame = 0
      O()\OFrame = FindMapElement(OF(), StringField(*OC\FrameNames, 1, "|"))
      O()\FTime = BeatTime
    EndIf
  Next
  
  LastElement(O())
  GenX = O()\x - #BG_BaseW + 100.0
  
  Tries + 1
  If (Tries > 1)
    ;FirstElement(O())
    ;DeleteElement(O())
  EndIf
  IntroTime = ElapsedMilliseconds()
  
  space = #True
  
EndProcedure

Procedure GenEnem()
  Static *OC.OBCLASS, *OF.OBFRAME, Scale.f, x.f, y.f, i.i, n.i, Throw.i
  
  x = GenX + #BG_BaseW + 100 + Random(Int(8000.0/Log(GenX)))
  
  If (x - LastThrow > #BG_BaseW) And (Random(3) = 0) And (Throw = #False)
    *OC = FindMapElement(OC(), "thrower")
    n = 2
    LastThrow = x
    Throw = #True
  Else
    n = ListSize(BC())
    SelectElement(BC(), Random(n-1))
    *OC = BC()
    
    If (*OC\Flags & #BG_Popup)
      n = 1
    Else
      n = 1 + Random(3)
    EndIf
    
    Throw = #False
  EndIf
  
  For i = 1 To n
    If (Throw) And (i = 2)
      Repeat
        SelectElement(WL(), Random(Weapons-1))
        *OC = WL()
      Until (*OC\Weapon <> *MyWeapon)
;       SelectElement(WL(), 1)
;       *OC = WL()
    EndIf
;     If (Throw)
;       Debug *OC\FrameNames
;     EndIf
    
    AddElement(O())
      O()\Class = *OC
      O()\Frame = 0
      O()\FTime = BeatTime
      O()\OFrame = FindMapElement(OF(), StringField(*OC\FrameNames, 1, "|"))
      *OF = O()\OFrame
      
      If (Throw)
        O()\sx = x + i * 5.0
        Scale = 1.0
      Else
        O()\sx = x + i * 50
        Scale = (70 + Random(100))/100.0
      EndIf
      O()\Scale = Scale
      
      O()\Flags = *OC\Flags | #BG_Gen
      If (O()\Flags & #BG_Popup)
        If (O()\Flags & #BG_RPopup)
          O()\ty = #BG_BaseH
          O()\sy = O()\ty + *OF\Size * Scale
        Else
          O()\ty = *OF\Size * Scale
          O()\sy = 0.0
        EndIf
      ElseIf (Throw)
        O()\ty = 0.0
        O()\sy = 0.0
      Else
        O()\ty = 1.0 * #BG_BaseH * (40 + Random(60)) / 100
        O()\sy = O()\ty
      EndIf
      
      If O()\Flags & #BG_Newton
        O()\yv = *OC\risespeed
      EndIf
      O()\HP = *OC\HP
      O()\Flags & (~(#BG_Dead|#BG_Awake))
      
      O()\x  = O()\sx
      O()\y  = O()\sy
      O()\py = O()\sy
      
      If (Throw) And (i = 1)
        O()\Mess = Shout(Random(#BG_Shouts - 1))
      Else
        O()\Mess = ""
      EndIf
      
      
  Next i
  GenX = O()\x - #BG_BaseW
    
EndProcedure

Procedure LoadLevel()
  Static St.s, Found.i, *OC.OBCLASS, x.f, w.f, i.i, *OF.OBFRAME
  
  ;SetWindowTitle(0, #BG_Title + " - Level " + Str(Level))
  
  ClearList(O())
  
  Restore Levels
  Found = #False
  Read.s St
  While (St <> #BG_DataEnd)
    If St = ";LEVEL" + Str(Level)
      
      Read.s St
      While (St)
        St = LCase(St)
        AddElement(O())
          *OC = FindMapElement(OC(), St)
          O()\Class = *OC
          *OF = FindMapElement(OF(), StringField(*OC\FrameNames, 1, "|"))
          Read.s O()\Mess
          
          Read.f O()\sx
          Read.f O()\ty
          Read.f O()\Scale
          O()\Flags = *OC\Flags
          If (O()\Flags & #BG_Popup)
            If (O()\Flags & #BG_RPopup)
              O()\sy = #BG_BaseH + *OF\Size * O()\Scale
            Else
              O()\sy = 0.0
            EndIf
          Else
            O()\sy = O()\ty
          EndIf
        Read.s St
      Wend
      
      GenX = O()\sx - #BG_BaseW
      
      Found = #True
      Break
    EndIf
    Read.s St
  Wend
  
  InIntro = #True
  
  SortStructuredList(O(), #PB_Sort_Ascending, OffsetOf(OBSTACLE\sx), #PB_Sort_Float)
  
  ;RandomSeed(0)
  x = 0.0
  For i = 1 To 10
    AddElement(BD())
      BD()\Color = RGB(Random(192),Random(192),Random(192))
      w = 100 + Random(300)
      BD()\w = w
      BD()\h = Random(ScreenH*8/10)
      BD()\x = x + w + 100.0 + Random(400)
      BD()\z = #BG_BaseW + Random(900)
      x = BD()\x
  Next i
  SortStructuredList(BD(), #PB_Sort_Descending, OffsetOf(BUILDING\z), #PB_Sort_Float)
  ;RandomSeed(ElapsedMilliseconds())
  
  ResetLevel()
EndProcedure

Procedure Shoot()
  Static *OF.OBFRAME, f.f

  f = *MyWeapon\BSpeed * (1.0 + Random(*MyWeapon\Rand)/20.0)

  AddElement(B())
    *OF = FindMapElement(OF(), StringField(*MyWeapon\FrameNames, 1, "|"))
    B()\OFrame  = *OF
    B()\x = bx + sox + Cos(Radian(pa))*sor
    B()\y = by - soy - Sin(Radian(pa))*sor
    B()\xv = CamV + f * Cos(Radian(pa))
    B()\yv = f * Sin(-Radian(pa))
    B()\Weapon = *MyWeapon
    B()\Frame = 0
    B()\FTime = Time
    B()\a = pa
    
  pxv - *MyWeapon\Recoil
    
  If (*MyWeapon\ShootSound)
    CueSound(*MyWeapon\ShootSound)
  EndIf
  
  If (MyAmmo > 0)
    MyAmmo - 1
    If (MyAmmo = 0)
      *MyWeapon = #Null
      CueSound(PopSound)
    EndIf
  EndIf
  
  ShootTime = Time
EndProcedure

Procedure AddHealth(Amount.i = 1)
  Static i.i, j.i, x.f, y.f
  LastElement(SN())
  x = SN()\x
  y = SN()\y
  For j = 1 To Amount
    For i = 1 To #BG_SpringNodes
      AddElement(SN())
        SN()\x = x
        SN()\y = y
      Nodes + 1
    Next i
  Next j
  Health + Amount
EndProcedure

Procedure Impact(*B.BULLET, *O.OBSTACLE)
  Static *W.WEAPON, *OC.OBCLASS
  
  *W = *B\Weapon
  *OC = *O\Class
  
  *O\HP - *W\Damage
  If (*O\HP <= 0)
    *O\Flags | #BG_Dead
    If (*OC\DieSound)
      CueSound(*OC\DieSound)
    EndIf
    Kills + 1
    If (Kills % 25 = 0)
      CueSound(Woo)
      CueMessage(Str(Kills) + " kills! More health!")
      AddHealth(2)
    EndIf
  Else
    *O\x + B()\xv;*OC\xv * 8.0
    If (*OC\HitSound)
      CueSound(*OC\HitSound)
    EndIf
  EndIf
  B()\Hits + 1
  If (B()\Hits >= *W\Hits)
    DeleteElement(B())
  EndIf
EndProcedure

Procedure Recolor(Color.i)
  Static x.i, y.i, dark.i, Buffer.i, Pitch.i, *L.LONG
  
  CopySprite(#BG_Reddy0, #BG_Reddy, #PB_Sprite_Texture)
  If (Color >= 0)
    MyColor = Color
    dark = RGB(Red(Color)*7/8, Green(Color)*7/8, Blue(Color)*7/8)
    If (StartDrawing(SpriteOutput(#BG_Reddy)))
      If (DrawingBufferPixelFormat() = #PB_PixelFormat_32Bits_BGR) ; BGR (swapped) recolor
        Color = RGB(Blue(Color),Green(Color),Red(Color))
        dark = RGB(Red(Color)*7/8, Green(Color)*7/8, Blue(Color)*7/8)
        Buffer = DrawingBuffer()
        Pitch  = DrawingBufferPitch()
        For y = 0 To #BG_RedSize - 1
          *L = Buffer + y * Pitch
          For x = 0 To #BG_RedSize - 1
            If (*L\l & $FFFFFF = $FF0000)
              *L\l = Color
            ElseIf (*L\l & $FFFFFF = $E12525)
              *L\l = Dark
            EndIf
            *L + 4
          Next x
        Next y
      ElseIf (DrawingBufferPixelFormat() = #PB_PixelFormat_32Bits_RGB) ; RGB (normal) recolor
        Buffer = DrawingBuffer()
        Pitch  = DrawingBufferPitch()
        For y = 0 To #BG_RedSize - 1
          *L = Buffer + y * Pitch
          For x = 0 To #BG_RedSize - 1
            If (*L\l & $FFFFFF = $0000FF)
              *L\l = Color
            ElseIf (*L\l & $FFFFFF = $2525E1)
              *L\l = Dark
            EndIf
            *L + 4
          Next x
        Next y
      Else ; Standard (slow) recolor
        For y = 0 To #BG_RedSize - 1
          For x = 0 To #BG_RedSize - 1
            Select (Point(x,y))
              Case $0000FF
                Plot(x,y,Color)
              Case $2525E1
                Plot(x,y,Dark)
            EndSelect
          Next x
        Next y
      EndIf
      StopDrawing()
    EndIf
  EndIf
  TransparentSpriteColor(#BG_Reddy, #BG_TransColor)
  If (Not IsSprite3D(#BG_3DReddy))
    CreateSprite3D(#BG_3DReddy, #BG_Reddy)
    TransparentSpriteColor(#BG_Reddy, #BG_TransColor)
    ZoomSprite3D(#BG_3DReddy, SpriteWidth(#BG_Reddy)*YScale, SpriteHeight(#BG_Reddy)*YScale)
  EndIf
EndProcedure









;-
;- Initializations

If (Not InitSprite())
  Error("Screen could not be initialized!")
EndIf

If (Not InitSprite3D())
  Error("Sprite3D could not be initalized!")
EndIf

If (Not InitKeyboard())
  Error("Keyboard input could not be initialized!")
EndIf

UsePNGImageEncoder()
UsePNGImageDecoder()

CompilerIf ((#PB_Compiler_OS = #PB_OS_MacOS) And (#PB_Compiler_Debugger = #False))
  SetCurrentDirectory(GetPathPart(ProgramFilename()) + "../../../")
CompilerEndIf

If (FileSize("see") = -1)
  SetCurrentDirectory("..")
EndIf

If (ReadFile(0, #BG_SaveFile))
  St = StringField(ReadString(0), 1, " ")
  St2 = ReadString(0)
  If (St2 = Hash(St))
    RecordReach = ValF(St)
  EndIf
  St = ReadString(0)
  If (Len(St) = 6)
    MyColor = Val("$" + St)
    MyColor = RGB(Blue(MyColor),Green(MyColor),Red(MyColor))
  EndIf
  CloseFile(0)
EndIf

SoundInit = InitSound()
If (SoundInit)
  UseOGGSoundDecoder()
  SoundOn = #True
  MusicOn = #True
  If (FileSize(#BG_HearPath + #BG_Music2) > 0)
    LoadSound(0, #BG_HearPath + #BG_Music2, #BG_MusicFlags)
    BG_MusicLen = 205000
  Else
    LoadSound(0, #BG_HearPath + #BG_Music, #BG_MusicFlags)
  EndIf
EndIf

JoyInit = InitJoystick()

FFont = LoadFont(0, "Arial", ScreenH * 0.03)


; Commandline

Windowed = #BG_DefWin

i = 0
n = CountProgramParameters()
While (i < n)
  Select LCase(ProgramParameter(i))
    Case "-f" : Delay(500) : Windowed = #False
    Case "-w" : Delay(500)
    Case "-i" : Immortal = #True
    Case "-c" : i + 1 : MyColor = Val("$" + ProgramParameter(i))
  EndSelect
  i + 1
Wend




; Open Screen

If (Windowed)
  k = #PB_Window_MinimizeGadget|#PB_Window_NoGadgets|#PB_Window_ScreenCentered
  If (Not OpenWindow(0, 0, 0, ScreenW, ScreenH, #BG_Title + ": Shift-F for Fullscreen", k))
    Error("A " + Str(ScreenW) + "x" + Str(ScreenH) + " window could not be opened!")
  EndIf
  If (Not OpenWindowedScreen(WindowID(0), 0, 0, ScreenW, ScreenH, 0, 0, 0))
    Error("A " + Str(ScreenW) + "x" + Str(ScreenH) + " windowed screen could not be opened!")
  EndIf
Else
  If (Not OpenScreen(ScreenW, ScreenH, ScreenD, #BG_Title))
    Error("A " + Str(ScreenW) + "x" + Str(ScreenH) + "x" + Str(ScreenD) + " screen could not be opened!")
  EndIf
EndIf

XScale = 1.0 * ScreenW / #BG_BaseW
YScale = 1.0 * ScreenH / #BG_BaseH

InScreen = #True
ClearScreen(#Black)
FlipBuffers()





; Load Balloon Character

LoadSprite(#BG_Reddy0, "./see/reddy2.png", #PB_Sprite_Texture)
  Recolor(MyColor)






; Scan Object Sprites

Restore ObFrames
Read.s St
While (St)
  St = LCase(St)
  AddMapElement(OF(), St)
    OF()\Sprite2D = LoadSprite(#PB_Any, "./see/" + St + ".png", #PB_Sprite_Texture)
    TransparentSpriteColor(OF()\Sprite2D, #BG_TransColor)
    OF()\Sprite3D = CreateSprite3D(#PB_Any, OF()\Sprite2D)
    TransparentSpriteColor(OF()\Sprite2D, #BG_TransColor)
    OF()\Size = SpriteHeight(OF()\Sprite2D)
    ZoomSprite3D(OF()\Sprite3D, OF()\Size * YScale, OF()\Size * YScale)
  Read.s St
Wend


; Scan Sounds
If (SoundInit)
  Restore SFX
  Read.s St
  While (St)
    St = LCase(St)
    AddMapElement(S(), St)
      S() = LoadSound(#PB_Any, #BG_HearPath + St + ".wav")
    Read.s St
  Wend
  PopSound = S("pop")
  RecSound = S("rec")
  SadSound = S("sad")
  WindSound = S("wind")
  TypeSound = S("type")
  Wahoo = S("wahoo")
  Ohh = S("ohh")
  Woo = S("woo")
EndIf



; Scan Weapons
Restore Weapons
Read.s St
While (St)
  St = LCase(St)
  AddMapElement(W(), St)
    Read.s St
    W()\OFrame = FindMapElement(OF(), LCase(St))
    Read.s St
    St = LCase(St)
    W()\FrameNames = St
    W()\Frames = CountString(St, "|") + 1
    
    Read.s St
    W()\GetSound   = S(St)
    Read.s St
    W()\ShootSound = S(St)
    
    Read.f W()\BSpeed
    Read.f f
    W()\FrameTicks = #BG_BeatLen / f
    Read.f W()\Gravity
    Read.f W()\Recoil
    Read.i W()\Ammo
    Read.i W()\Rate
    Read.i W()\Limit
    Read.i W()\Damage
    Read.i W()\Hits
    Read.i W()\Rand
  Read.s St
Wend





; Scan Object Classes

Restore ObClasses
Read.s St
While (St)
  St = LCase(St)
  AddMapElement(OC(), St)
    Read.s St
    St = LCase(St)
    OC()\FrameNames = St
    OC()\Frames = CountString(St, "|") + 1
    FindMapElement(OF(), StringField(St, 1, "|"))
    OC()\Size = OF()\Size
    
    Read.s St
    OC()\Flags = #Null
    St = LCase(St)
    If (St)
      OC()\Weapon = FindMapElement(W(), St)
      If (OC()\Weapon)
        OC()\Flags  = #BG_Weapon
        AddElement(WL())
          WL() = @OC()
        Weapons + 1
      EndIf
    EndIf
    
    Read.s St
    OC()\AppearSound = S(St)
    Read.s St
    OC()\HitSound = S(St)
    Read.s St
    OC()\DieSound = S(St)
    
    Read.f f
    OC()\FrameTicks = #BG_BeatLen / f
    Read.f OC()\xv
    Read.f OC()\hoveramount
    Read.f OC()\hoverspeed
    Read.f OC()\popspeed
    
    Read.f OC()\oleft
    Read.f OC()\oright
    Read.f OC()\otop
    Read.f OC()\obottom
    Read.f OC()\hide
    
    Read.i OC()\HP
    Read.i k
    OC()\Flags | k
    
    If (Not (OC()\Flags & #BG_Newton))
      OC()\hoverspeed = OC()\hoverspeed * 0.1
    EndIf
    
    If (OC()\Flags & #BG_Bad)
      AddElement(BC())
        BC() = @OC()
    EndIf
  Read.s St
Wend






; Calculate Offsets

MinX =   0.0 + 10.0
MaxX = #BG_BaseW - 10.0 - #BG_RedSize
MinY =   0.0 + #BG_RedSize + 15.0
MaxY = #BG_BaseH - 15.0

sox = SpriteWidth(#BG_Reddy)  * 0.5
soy = SpriteHeight(#BG_Reddy) * 0.5
sor = SpriteHeight(#BG_Reddy) * 0.4










;- Begin

BackR = 255
BackG = 255
BackB = 255

Level = 1
LoadLevel()

Repeat

  ; Window Events

  If (Windowed)
    Event = WindowEvent()
    While (Event)
      If (Event = #PB_Event_CloseWindow)
        ExitFlag = #True
      EndIf
      Event = WindowEvent()
    Wend
  EndIf
  
  
  
  ;- Controls
  
  ExamineKeyboard()
  If (JoyInit)
    ExamineJoystick(0)
  EndIf
  
  ; Intro
  
  If (InIntro)
    px = 690.0 - CamX
    If (px <= #BG_BaseW/2)
      StartX = px
      InIntro = #False
      CueMusic()
      ;CueSound(Woo)
      CueSound(WindSound)
    EndIf
  Else
    
    If (Time - BeatTime > BG_MusicLen) And (Not Dead)
      CueMusic()
    EndIf
    
      If (Not Dead)
      i = KeyboardPushed(#PB_Key_Left)
      j = KeyboardPushed(#PB_Key_Right)
      
      If (JoyInit)
        If JoystickAxisX(0) = -1
          i = 1
        ElseIf JoystickAxisX(0) = 1
          j = 1
        EndIf
      EndIf
      
      If (i)
        If (LastXKey = -1) And (Time - LastXTime < #BG_TapTime)
          DoubleX = 3.0
        EndIf
        LastXKey = -1
        LastXTime = 0
        pxv - 0.3 * DoubleX
      ElseIf (j)
        If (LastXKey = 1) And (Time - LastXTime < #BG_TapTime)
          DoubleX = 3.0
        EndIf
        LastXKey = 1
        LastXTime = 0
        pxv + 0.3 * DoubleX
      Else
        pxv * 0.95
        DoubleX = 1.0
      EndIf
      If (KeyboardReleased(#PB_Key_Left) Or KeyboardReleased(#PB_Key_Right))
        LastXTime = Time
      EndIf
      
      i = KeyboardPushed(#PB_Key_Up)
      j = KeyboardPushed(#PB_Key_Down)
      
      If (JoyInit)
        If JoystickAxisY(0) = -1
          i = 1
        ElseIf JoystickAxisY(0) = 1
          j = 1
        EndIf
      EndIf
      
      If (i)
        If (LastYKey = 1) And (Time - LastYTime < #BG_TapTime)
          DoubleY = 2.0
        EndIf
        LastYKey = 1
        LastYTime = 0
        pyv + 0.2 * DoubleY
      ElseIf (j)
        If (LastYKey = -1) And (Time - LastYTime < #BG_TapTime)
          DoubleY = 2.0
        EndIf
        LastYKey = -1
        LastYTime = 0
        pyv - 0.2 * DoubleY
      Else
        pyv = pyv * 0.95
        DoubleY = 1.0
        ;pyv = pyv + (-0.1 - pyv) * 0.05
      EndIf
      If (KeyboardReleased(#PB_Key_Up) Or KeyboardReleased(#PB_Key_Down))
        LastYTime = Time
      EndIf
      
    EndIf
    
    If (pxv > #BG_MaxSpeed)
      pxv = #BG_MaxSpeed
    ElseIf (pxv < -#BG_MaxSpeed)
      pxv = -#BG_MaxSpeed
    EndIf
    
    px     + pxv
    py     + pyv
    drifta + 0.01
    
    ; Screen Boundaries
    
    If (px < MinX)
      px = MinX
      If 0;(pxv < 0.0)
        pxv = 0.0
      EndIf
    ElseIf (px > MaxX)
      px = MaxX
      If 0;(pxv > 0.0)
        pxv = 0.0
      EndIf
    EndIf
    If (py < MinY)
      py = MinY
      If (pyv < 0.0)
        pyv = 0.0
      EndIf
    ElseIf (py > MaxY)
      py = MaxY
      If (pyv > 0.0)
        pyv = 0.0
      EndIf
    EndIf
  EndIf
  
  
  
  
  ; Move Camera, Update Balloon
  
  CamTV = (Int(Log10(bx)) + 1)
  If ((Not Dead) And ((Not InIntro) Or (Time - IntroTime > 2500)))
    CamV = CamV + (CamTV - CamV) * 0.02
    CamTX + CamV
  EndIf
  CamX = CamX + (CamTX - CamX) * 0.1
  
  If (CamX > GenX)
    GenEnem()
  EndIf
  
  pa = pa + (5.0 * pxv - pa) * 0.1
  bx = px + 10.0 * Cos(drifta * 0.5) + CamX
  by = py + 10.0 * Sin(drifta)
  
  
  
  
  
  
  ; Shoot!
  If (Not Dead) And (Not InIntro)
    ospace = space
    space  = KeyboardPushed(#PB_Key_LeftControl) Or KeyboardReleased(#PB_Key_RightControl)
    If (JoyInit)
      If (JoystickButton(0,1) Or JoystickButton(0,2))
        space = 1
      EndIf
    EndIf
    If (space And *MyWeapon)
      If (ospace)
        If (*MyWeapon\Rate)
          If (Time - ShootTime > *MyWeapon\Rate)
            Shoot()
          EndIf
        EndIf
      ElseIf (Time - ShootTime > *MyWeapon\Limit)
        Shoot()
      EndIf
    Else
    EndIf
  EndIf
  
  
  
  
  ; Bullet Physics
  ForEach B()
    *W = B()\Weapon
    B()\x + B()\xv
    B()\y + B()\yv
    B()\yv - *W\Gravity
    If (Time - B()\FTime > *W\FrameTicks)
      B()\FTime = Time
      B()\Frame = (B()\Frame + 1) % *W\Frames
      B()\OFrame = FindMapElement(OF(), StringField(*W\FrameNames, B()\Frame + 1, "|"))
    EndIf
    
    k = #False
    If (B()\x > CamX)
      If (B()\x < CamX + #BG_BaseW)
        If (B()\y > 0.0)
          If (B()\y < #BG_BaseH)
            k = #True
          EndIf
        EndIf
      EndIf
    EndIf
    
    If (k)
      ForEach O()
        If (O()\Flags & #BG_Bad)
          If (Not (O()\Flags & #BG_Dead))
            *OC = O()\Class
            If (B()\x > O()\x + *OC\oleft * O()\Scale)
              If (B()\x < O()\x + (*OC\Size - *OC\oright) * O()\Scale)
                If (B()\y > O()\y - (*OC\Size - *OC\obottom) * O()\Scale)
                  If (B()\y < O()\y - *OC\otop * O()\Scale)
                    Impact(@B(), @O())
                    Break
                  EndIf
                EndIf
              EndIf
            EndIf
          EndIf
        EndIf
      Next
    Else
      DeleteElement(B())
    EndIf
  Next
  
  
  
  
  
  ; Balloon String Physics
  
  CompilerIf (#BG_SpringPhys)
    FirstElement(SN())
      SN()\x = bx + sox - Sin(Radian(pa)) * sor
      SN()\y = by - soy - Cos(Radian(pa)) * sor
      x = SN()\x
      y = SN()\y
    While (NextElement(SN()))
      dx = x - SN()\x
      dy = y - SN()\y
      d  = Sqr(dx*dx + dy*dy) - #BG_SpringLength
      a  = ATan2(dx, dy)
      f  = #BG_SpringConst * d
      SN()\xv = f * Cos(a)
      SN()\yv = f * Sin(a) - #BG_SpringGrav - Random(100.0*Int(CamV))*0.01
      SN()\x  + SN()\xv
      SN()\y  + SN()\yv
      x = SN()\x
      y = SN()\y
    Wend
    If (Hurt And (Not Immortal))
      LastElement(SN())
      For i = 1 To #BG_SpringNodes
        If (Nodes > 0)
          DeleteElement(SN())
          Nodes - 1
        EndIf
      Next i
    EndIf
  CompilerEndIf
  
  
  
  
  
  
  
  
  ; Obstacle Logic
  Hurt = #False
  
  Time = ElapsedMilliseconds()
  n = ((Time - BeatTime) % #BG_BeatLen)
  f = BeatPhase
  BeatPhase = n / #BG_BeatLen
  If (Not InIntro) And (f > BeatPhase)
    Repeat
      BackR = Random(1)
      BackG = Random(1)
      BackB = Random(1)
    Until (BackR + BackG + BackB)
    BackR = 205 + BackR * 50
    BackG = 205 + BackG * 50
    BackB = 205 + BackB * 50
  EndIf
  
  j = #False
  ForEach O()
    If (Not (O()\Flags & #BG_Dead))
      *OC = O()\Class
      If (O()\x > CamX + #BG_BaseW)
        If (Not (O()\Flags & #BG_Awake))
          Break
        EndIf
      ElseIf (O()\x + *OC\Size * O()\Scale < CamX)
        If (O()\Flags & #BG_Volatile)
          O()\Flags | #BG_Dead
        EndIf
      Else
;         If (O()\Flags & #BG_Weapon)
;           ;Debug "Awake: " + Str(0 Or O()\Flags & #BG_Awake)
;           ;Debug O()\x
;           ;Debug ""
;           ;Debug "      wake up!"
;         EndIf
        If (Not (O()\Flags & #BG_Awake))
          If (O()\x < CamX + #BG_BaseW - *OC\hide)
            O()\Flags | #BG_Awake
            If (*OC\AppearSound)
              CueSound(*OC\AppearSound)
            EndIf
            If (O()\Mess) And (Not Dead)
              CueMessage(O()\Mess)
            EndIf
          EndIf
        EndIf
      EndIf
      If (O()\Flags & #BG_Awake)
        O()\x - *OC\xv
        If (*OC\Flags & #BG_Popup)
          O()\py = O()\py + (O()\ty - O()\py) * *OC\popspeed
          O()\hovera + *OC\hoverspeed
          O()\y = O()\py + *OC\hoveramount * Sin(O()\hovera)
        ElseIf (O()\Flags & #BG_Newton)
          O()\yv - *OC\gravity
          O()\py + O()\yv
          O()\y = O()\py
        Else
          O()\hovera + *OC\hoverspeed
          O()\y = O()\py + *OC\hoveramount * Sin(O()\hovera)
        EndIf
      EndIf
      If (*OC\Frames > 1)
        k = O()\FTime
        O()\FTime = (Time - BeatTime) % *OC\FrameTicks
        If (k > O()\FTime)
          O()\Frame = (O()\Frame + 1) % *OC\Frames
          O()\OFrame = FindMapElement(OF(), StringField(*OC\FrameNames, O()\Frame + 1, "|"))
        EndIf
      EndIf
      
      ; Balloon/Obstacle Collision
      
      If (Not Dead)
        If (bx + #BG_RedSize > O()\x + *OC\oleft * O()\Scale)
          If (bx < O()\x + (*OC\Size - *OC\oright) * O()\Scale)
            If (by > O()\y - (*OC\Size - *OC\obottom) * O()\Scale)
              If (by - #BG_RedSize < O()\y - *OC\otop * O()\Scale)
                If (O()\Flags & #BG_Weapon)
                  *MyWeapon = *OC\Weapon
                  MyAmmo = *MyWeapon\Ammo
                  If (*MyWeapon\GetSound)
                    CueSound(*MyWeapon\GetSound)
                  EndIf
                  O()\Flags | #BG_Dead
                ElseIf (O()\Flags & #BG_Bad)
                  Hurt = #True
                EndIf
              EndIf
            EndIf
          EndIf
        EndIf
      EndIf
    EndIf
  Next
  If (Hurt)
    pxv = -#BG_MaxSpeed*0.5
    CamTX - #BG_BaseW*0.5
    CueSound(PopSound)
    If (Not Immortal)
      Health - 1
      If (Health <= 0)
        ;ResetLevel()
        Dead = #True
        CueSound(Ohh)
        DeadTime = Time + 5000
        If (Record)
          CueMessage("New record: " + StrF(RecordReach, 1) + " ABU!")
        Else
          CueMessage("Survived " + StrF(Reach, 1) + " ABU!")
        EndIf
      EndIf
    EndIf
  EndIf
  If (Dead)
    If (Time > DeadTime)
      ;ResetLevel()
      CueMessage("Press R to try again.")
      DeadTime + 10000
    EndIf
  EndIf
  
  
  
  
  
  ; Reach
  If (Not Dead)
    x = (px + CamX - #BG_StartX) / 100.0
    If (x > Reach)
      Reach = x
      If (Reach > RecordReach)
        If (Not Record)
          Record = #True
          If (RecordReach > 0.0)
            CueSound(WindSound)
            CueSound(Wahoo)
          EndIf
        EndIf
        RecordReach = Reach
      EndIf
    EndIf
  EndIf
  
  
  
  
  
  
  
  
  
  ;- Render
  
  f = (1.0 - BeatPhase*0.1)
  If (InIntro)
    ClearScreen(#White)
  Else
    k = RGB(BackR * f, BackG * f, BackB * f)
    ClearScreen(k)
  EndIf
  
  If (StartDrawing(ScreenOutput()))
    
    ForEach BD()
      x = ScreenW/2 + #BG_BaseW * (BD()\x - CamX)/BD()\z
      n = #BG_BaseW * (BD()\w) / BD()\z
      If (x < ScreenW)
        If (x + n < 0)
          BD()\x + BD()\z*2 + Random(1800)
          BD()\Color = RGB(Random(192),Random(192),Random(192))
        Else
          Box(x, ScreenH - Int(BD()\h), n, Int(BD()\h), BD()\Color)
        EndIf
      EndIf
    Next
    
    Box(0, ScreenH*9/10, ScreenW, ScreenH/10, $FFF0E0)
    
    If (Not InIntro) And (Not Dead)
      St = "Survived " + StrF(Reach, 1) + " ABU"
      i = ScreenW * 0.05
      j = ScreenH * 0.05
      If (SurviveY < j)
        SurviveY + 1
      EndIf
      DrawingFont(FFont)
      k = TextWidth(St)
      n = TextHeight(St)
      If (Record)
        Box(i - 5, SurviveY - 5, k + 10, n + 10, #Black)
        Box(i - 4, SurviveY - 4, k + 8, n + 8, RGB(BackB, BackR, BackG))
      EndIf
      DrawingMode(#PB_2DDrawing_Transparent)
      DrawText(i, SurviveY, St, #Black)
    EndIf
    If (InIntro)
      i = ScreenW * 0.01
      j = ScreenH * 0.95 + Int((CamX - 200.0)*0.5)
      DrawingMode(#PB_2DDrawing_Transparent)
      DrawingFont(FFont)
      DrawText(i, j, "v" + #BG_Version, $808080)
    EndIf
    
    CompilerIf (0);1)
      DrawingMode(#PB_2DDrawing_Outlined|#PB_2DDrawing_Transparent)
      d = ((1.0 - BeatPhase) * 0.1)
      f = 1.0 + d
      ForEach O()
        ;If (Not (O()\Flags & #BG_Dead))
          *OC = O()\Class
          *OF = O()\OFrame
          If (O()\x > CamX + #BG_BaseW)
            ;Break
          Else;If (Not (O()\Flags & #BG_Dead))
            If (O()\Flags & #BG_Awake)
              k = #White
            Else
              If (O()\Flags & #BG_Popup)
                k = #Green
              Else
                k = #Black
              EndIf
              DrawText(ScreenX(O()\x),ScreenY(O()\y), StrF(*OC\hide,1),k)
            EndIf
            Box(ScreenX(O()\x), ScreenY(O()\y + 20.0), *OF\Size * O()\Scale, *OF\Size * O()\Scale, k)
          EndIf
        ;EndIf
      Next
    CompilerEndIf
    
    CompilerIf (#BG_SpringPhys)
      FirstElement(SN())
        x = SN()\x
        y = SN()\y
      While (NextElement(SN()))
        LineXY(ScreenX(x),ScreenY(y),ScreenX(SN()\x),ScreenY(SN()\y),#Black)
        x = SN()\x
        y = SN()\y
      Wend
    CompilerEndIf
    StopDrawing()
  EndIf
  
  ; Sprite3D Render
  
  If (Start3D())
    
    If (Not Dead)
      ZoomSprite3D(#BG_3DReddy, SpriteWidth(#BG_Reddy)*YScale, SpriteHeight(#BG_Reddy)*YScale)
      RotateSprite3D(#BG_3DReddy, pa, #PB_Absolute)
      DisplaySprite3D(#BG_3DReddy, ScreenX(bx), ScreenY(by))
    EndIf
    
    If (*MyWeapon) And (Not Dead)
      *OF = *MyWeapon\OFrame
      f = *OF\Size * YScale
      ZoomSprite3D(*OF\Sprite3D, f, f)
      RotateSprite3D(*OF\Sprite3D, pa, #PB_Absolute)
      d = (*OF\Size - #BG_RedSize)*0.5
      DisplaySprite3D(*OF\Sprite3D, ScreenX(bx - d), ScreenY(by + d))
    EndIf
  
    d = ((1.0 - BeatPhase) * 0.1)
    f = 1.0 + d
    ForEach O()
      If (Not (O()\Flags & #BG_Dead))
        *OC = O()\Class
        *OF = O()\OFrame
        If (O()\x > CamX + #BG_BaseW)
          If (Not (O()\Flags & #BG_Awake))
            Break
          EndIf
        Else;If (Not (O()\Flags & #BG_Dead))
          If (O()\Flags & #BG_Newton)
            RotateSprite3D(*OF\Sprite3D, O()\yv, #PB_Absolute)
          Else
            RotateSprite3D(*OF\Sprite3D, 0.0, #PB_Absolute)
          EndIf
          If (O()\Flags & #BG_Pulsy)
            ZoomSprite3D(*OF\Sprite3D, *OF\Size * O()\Scale * f, *OF\Size * O()\Scale * f)
            DisplaySprite3D(*OF\Sprite3D, ScreenX(O()\x - *OF\Size * O()\Scale * d * 0.5), ScreenY(O()\y + *OF\Size * O()\Scale * d * 0.5))
          Else
            ZoomSprite3D(*OF\Sprite3D, *OF\Size * O()\Scale, *OF\Size * O()\Scale)
            DisplaySprite3D(*OF\Sprite3D, ScreenX(O()\x), ScreenY(O()\y))
          EndIf
        EndIf
      EndIf
    Next
    
    ForEach B()
      *W = B()\Weapon
      *OF = B()\OFrame
      RotateSprite3D(*OF\Sprite3D, B()\a, #PB_Absolute)
      DisplaySprite3D(*OF\Sprite3D, ScreenX(B()\x - *OF\Size*0.5), ScreenY(B()\y + *OF\Size*0.5))
    Next
  
    Stop3D()
    
    If (Message)
      If (Time < MessageTime) Or (MessageN < MessageLen)
        If (StartDrawing(ScreenOutput()))
          DrawingMode(#PB_2DDrawing_Transparent)
          DrawingFont(FFont)
          St = Left(Message, MessageN)
          k = TextWidth(St)
          n = TextHeight(St)
          i = ScreenW/2 - k/2
          j = ScreenH*2/3
          Box(i - 5, j - 5, k + 10, n + 10, #Black)
          Box(i - 4, j - 4, k + 8, n + 8, #White)
          DrawText(i,j,St,#Black)
          StopDrawing()
        EndIf
        If (MessageN < MessageLen)
          If (Random(3) = 0)
            MessageN + 1
            CueSound(TypeSound)
          EndIf
        EndIf
      Else
        Message = ""
      EndIf
    EndIf
  EndIf
  FlipBuffers()
  
;   If (KeyboardPushed(#PB_Key_Space))
;     Repeat : ExamineKeyboard() : Delay(100) : Until KeyboardPushed(#PB_Key_G)
;   EndIf
  
  If KeyboardReleased(#PB_Key_F9)
    k = GrabSprite(#PB_Any, 0, 0, ScreenW, ScreenH)
    If (k)
      CreateDirectory("./screens/")
      SaveSprite(k, "./screens/" + FormatDate("%yyyy%mm%dd%hh%ii%ss.png", Date()), #PB_ImagePlugin_PNG)
      FreeSprite(k)
    EndIf
  EndIf
  
  If KeyboardReleased(#PB_Key_R) And (Not InIntro)
    ;Tries = 0
    ResetLevel()
  EndIf
  
  If KeyboardReleased(#PB_Key_S)
    SoundOn = 1 - SoundOn
    If (SoundOn)
      CueMessage("Sound ON")
    Else
      CueMessage("Sound OFF")
    EndIf
  EndIf
  
  If KeyboardReleased(#PB_Key_M)
    MusicOn = 1 - MusicOn
    If (MusicOn)
      SoundVolume(0, 100)
      CueMessage("Music ON")
    Else
      SoundVolume(0, 0)
      CueMessage("Music OFF")
    EndIf
  EndIf
  
  
  
  
  ; Toggle Screen
  If (KeyboardPushed(#PB_Key_LeftShift) Or KeyboardPushed(#PB_Key_RightShift))
    If (KeyboardReleased(#PB_Key_F))
      Shutdown(#True)
    ElseIf (KeyboardReleased(#PB_Key_I))
      Immortal = (Immortal ! 1) & 1
      If (Immortal)
        CueMessage("Immortality ON")
      Else
        CueMessage("Immortality OFF")
      EndIf
    EndIf
  EndIf
  
  ; Choose Color
  i = KeyboardPushed(#PB_Key_Space)
  If (i)
    If (Not ospkey) And (Not Dead)
      Recolor(Random(#White))
      CueSound(PopSound)
    EndIf
  EndIf
  ospkey = i
  If (KeyboardReleased(#PB_Key_C) And Windowed)
    i = ColorRequester(MyColor)
    If (i >= 0)
      Recolor(i)
    EndIf
  EndIf
  
  
  
  
  
  ; Exit Loop
  
  If (KeyboardPushed(#PB_Key_Escape))
    ExitFlag = #True
  EndIf
  
  Delay(10)
Until (ExitFlag)
Shutdown()













;- Datasection

DataSection
  
  SFX:
  Data.s "pop"
  Data.s "bull"
  Data.s "hit"
  Data.s "hitt"
  Data.s "ac"
  Data.s "ace"
  Data.s "uke"
  Data.s "metal"
  Data.s "rec"
  Data.s "sad"
  Data.s "wind"
  Data.s "type"
  Data.s "harm"
  Data.s "aww"
  Data.s "upp"
  Data.s "caw"
  Data.s "cawd"
  Data.s "shoo"
  Data.s "shot"
  Data.s "qui"
  Data.s "ting"
  Data.s "bleh"
  Data.s "splat"
  Data.s "bzz"
  Data.s "grr"
  Data.s "wha"
  Data.s "wahoo"
  Data.s "woo"
  Data.s "ohh"
  Data.s ""
  
  ObFrames:
  Data.s "throw1"
  Data.s "stand"
  
  Data.s "sbull1", "sbull2"
  Data.s "bull1", "bull2"
  Data.s "stone"
  Data.s "tart"
  Data.s "rbull"
  Data.s "nails", "nails2"
  Data.s "bowlb"
  Data.s "waterb"
  Data.s "fire"
  
  Data.s "cact1", "cact2"
  Data.s "came1", "came2"
  Data.s "stop1", "stop2"
  Data.s "pots1", "pots2"
  Data.s "bbird1","bbird2"
  Data.s "rbird1","rbird2"
  Data.s "bee1","bee2"
  Data.s "utree"
  Data.s "pcloud"
  
  Data.s "pistol"
  Data.s "rifle"
  Data.s "sling"
  Data.s "toaster"
  Data.s "chain"
  Data.s "bucket"
  Data.s "bowlw"
  Data.s "waterm"
  Data.s "flamet"
  
  Data.s "logo"
  Data.s ""
  
  
  
  
  
  
  
  Weapons:
  Data.s "pistol", "pistol", "sbull1|sbull2"  ; name frame bframes
    Data.s "ac", "shoo"                     ; getsound shootsound
    Data.f 12.0, 4.0, 0.0, 1.0              ; bspeed fspeed grav recoil
    Data.i 12, 0, 0, 10, 1, 0               ; ammo rate limit damage hits random
  Data.s "rifle", "rifle", "bull1|bull2"
    Data.s "ace", "shot"
    Data.f 10.0, 6.0, 0.0, 4.0
    Data.i 16, 0, 300, 20, 1, 0
  Data.s "sling", "sling", "stone"
    Data.s "uke", "bull"
    Data.f 8.0, 1.0, 0.04, 0.0
    Data.i 24, 400, 100, 10, 1, 0
  Data.s "toaster", "toaster", "tart"
    Data.s "uke", "metal"
    Data.f 10.0, 1.0, -0.05, 0.05
    Data.i 10, 500, 500, 30, 2, 0
  Data.s "chaingun", "chain", "rbull"
    Data.s "ace", "bull"
    Data.f 12.0, 1.0, 0.0, 1.0
    Data.i 30, 100, 100, 10, 1, 0
  Data.s "bucket", "bucket", "nails|nails2"
    Data.s "ac", "bull"
    Data.f 4.0, 1.538, 1.5, 0.0
    Data.i 50, 50, 100, 5, 1, 20
  Data.s "bowl", "bowlw", "bowlb"
    Data.s "harm", "bull"
    Data.f 12.0, 1.0, 0.04, 10.0
    Data.i 1, 500, 100, 500, 3, 0
  Data.s "melon", "waterm", "waterb"
    Data.s "harm", "bull"
    Data.f 12.0, 1.0, 0.04, 10.0
    Data.i 1, 500, 100, 500, 10, 0
  Data.s "flame", "flamet", "fire"
    Data.s "ace", "bull"
    Data.f 6.0, 1.0, -0.08, 0.5
    Data.i 80, 50, 100, 5, 1, 20
  Data.s ""
  
  
  
  
  
  ObClasses:
  Data.s "cactus", "cact1|cact2", ""          ; name frames weapon
    Data.s "upp", "hit", "aww"                ; appear hit die
    Data.f 1.0, 0.0, 0.0, 0.0, 0.02           ; framefreq xv hoveramount hoverfreq/grav popspeed/yv0
    Data.f 26.0, 45.0, 0.0, 0.0, 100.0        ; left right top bottom hide
    Data.i 50, #BG_Bad|#BG_Popup|#BG_Pulsy    ; hp flags
  Data.s "camel", "came1|came2", ""
    Data.s "upp", "hit", "bleh"
    Data.f 1.0, 0.0, 0.0, 0.0, 0.05
    Data.f 10.0, 20.0, 10.0, 0.0, 100.0
    Data.i 40, #BG_Bad|#BG_Popup|#BG_Pulsy
  Data.s "stopsign", "stop1|stop2", ""
    Data.s "metal", "ting", "qui"
    Data.f 1.0, 0.0, 0.0, 0.0, 0.2
    Data.f 48.0, 36.0, 8.0, 0.0, 100.0
    Data.i 30, #BG_Bad|#BG_Popup|#BG_Pulsy
  Data.s "potssign", "pots1|pots2", ""
    Data.s "metal", "ting", "qui"
    Data.f 1.0, 0.0, 0.0, 0.0, 0.2
    Data.f 48.0, 36.0, 8.0, 0.0, 100.0
    Data.i 30, #BG_Bad|#BG_Popup|#BG_Pulsy
  Data.s "bbird", "bbird1|bbird2", ""
    Data.s "caw", "hitt", "cawd"
    Data.f 1.0, 1.0, 30.0, 1.0, 0.0
    Data.f 1.0, 0.0, 6.0, 10.0, 0.0
    Data.i 20, #BG_Bad
  Data.s "rbird", "rbird2|rbird1", ""
    Data.s "caw", "hit", "cawd"
    Data.f 1.0, 5.0, 30.0, 1.0, 0.0
    Data.f 1.0, 0.0, 6.0, 10.0, 0.0
    Data.i 20, #BG_Bad
  Data.s "bee", "bee1|bee2", ""
    Data.s "bzz", "hit", "splat"
    Data.f 2.0, 3.0, 20.0, 0.8, 0.0
    Data.f 6.0, 5.0, 12.0, 20.0, 0.0
    Data.i 10, #BG_Bad
  Data.s "pcloud", "pcloud", ""
    Data.s "grr", "hit", "wha"
    Data.f 1.0, 0.6, 40.0, 1.5, 0.0
    Data.f 14.0, 14.0, 21.0, 38.0, 0.0
    Data.i 15, #BG_Bad|#BG_Pulsy
  Data.s "utree", "utree", ""
    Data.s "metal", "hit", "qui"
    Data.f 1.0, 0.0, 0.0, 0.0, 0.2
    Data.f 27.0, 45.0, 0.0, 22.0, 100.0
    Data.i 60, #BG_Bad|#BG_Popup|#BG_RPopup|#BG_Pulsy
  
  Data.s "logo", "logo", ""
    Data.s "", "", ""
    Data.f 1.0, 0.0, 0.0, 0.0, 0.0
    Data.f 0.0, 0.0, 0.0, 0.0, 0.0
    Data.i 0, #BG_Friend|#BG_Volatile
  Data.s "stander", "stand", ""
    Data.s "", "", ""
    Data.f 1.0, 0.0, 0.0, 0.0, 0.0
    Data.f 0.0, 0.0, 0.0, 0.0, 500.0
    Data.i 0, #BG_Friend|#BG_Volatile
  Data.s "thrower", "throw1", ""
    Data.s "", "", ""
    Data.f 1.0, 0.0, 0.0, 0.0, 0.15
    Data.f 0.0, 0.0, 0.0, 0.0, 100.0
    Data.i 0, #BG_Popup|#BG_Pulsy|#BG_Friend|#BG_Volatile
  
  Data.s "pistol", "pistol", "pistol"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 60.0, 22.0, 54.0, 38.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "rifle", "rifle", "rifle"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 26.0, 7.0, 62.0, 40.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "sling", "sling", "sling"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 59.0, 20.0, 55.0, 35.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "toaster", "toaster", "toaster"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 63.0, 20.0, 55.0, 27.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "chaingun", "chain", "chaingun"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 36.0, 11.0, 62.0, 30.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "bucket", "bucket", "bucket"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 50.0, 25.0, 63.0, 30.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "bowl", "bowlw", "bowl"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 66.0, 23.0, 59.0, 30.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "melon", "waterm", "melon"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 45.0, 6.0, 66.0, 11.0, 100.0
    Data.i 0, #BG_Newton
  Data.s "flame", "flamet", "flame"
    Data.s "", "", ""
    Data.f 1.0, 1.0, 0.0, 0.1, 9.0
    Data.f 3.0, 26.0, 58.0, 29.0, 100.0
    Data.i 0, #BG_Newton
  Data.s ""
  
  
  
  
  
  
  
  
  
  
  Levels:
  Data.s ";LEVEL1"
  Data.s "logo", ""
    Data.f 320.0, 368.0, 1.0
  Data.s "stander", "Hey!"
    Data.f 640.0, 128.0, 1.0
  Data.s "thrower", "It's dangerous to go alone!"
    Data.f 1500.0, 128.0, 1.0
  Data.s "thrower", "Take this!"
    Data.f 2200.0, 128.0, 1.0
  Data.s "sling", ""
    Data.f 2200.0001, 0.0, 1.0
  
  Data.s "stopsign", "(Press Control to shoot!)"
    Data.f 2800.0, 192.0, 1.5
  Data.s "bbird", ""
    Data.f 3500.0, 350.0, 1.0
  Data.s "bbird", ""
    Data.f 3600.0, 300.0, 1.0
  Data.s "thrower", "Pay attention to your string!"
    Data.f 4000.0, 128.0, 1.0
  Data.s "cactus", "(Double-tap to move faster!)"
    Data.f 5000.0, 192.0, 1.5
    
  Data.s ""
  
  Data.s #BG_DataEnd
  
EndDataSection


;-
;  EOF
; IDE Options = PureBasic 4.51 (Windows - x86)
; CursorPosition = 1640
; FirstLine = 1637
; Folding = ---
; Executable = ..\Loonadrift.exe
; DisableDebugger